package org.zheng.demo.assertJsonCollection;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

public class TestAssertJsonCollectionWithJackson extends TestJsonData {

	ObjectMapper ob = new ObjectMapper();

	private JsonNode[] readJsonNodes(final String jsonArray) {
		JsonNode[] jsonNodeList = null;
		try {
			jsonNodeList = ob.readValue(jsonArray, JsonNode[].class);

		} catch (JsonProcessingException e) {

			e.printStackTrace();
		}

		return jsonNodeList;
	}

	@Test
	void test_readValue_JsonNode() {
		// parse JSON into Jackson'sJsonNode
		JsonNode[] actualJsonNodes = readJsonNodes(actualJsonArray);
		JsonNode[] expectedJsonNodes = readJsonNodes(expectedJsonArray);

		// sort the JsonNode array
		Arrays.sort(actualJsonNodes, Comparator.comparingInt(jNode -> jNode.get("id").asInt()));
		Arrays.sort(expectedJsonNodes, Comparator.comparingInt(jNode -> jNode.get("id").asInt()));

		// compare the JsonNode array without any order
		assertTrue(Arrays.equals(actualJsonNodes, expectedJsonNodes));
		assertTrue(Arrays.deepEquals(actualJsonNodes, expectedJsonNodes));

	}

	@Test
	void test_readValue_listMap() throws JsonMappingException, JsonProcessingException {
		List<Map<String, Object>> actualListMap = ob.readValue(actualJsonArray,
				new TypeReference<List<Map<String, Object>>>() {
				});

		List<Map<String, Object>> expectedListMap = ob.readValue(expectedJsonArray,
				new TypeReference<List<Map<String, Object>>>() {
				});

		List<Map<String, Object>> sortedExpected = expectedListMap.stream()
				.sorted(Comparator.comparing(map -> (Integer) map.get("id"))).toList();

		assertEquals(actualListMap, sortedExpected);

	}

}
